/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: AC3AudioStream.h,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef AC3AUDIOSTREAM_H
#define AC3AUDIOSTREAM_H

#include <deque>

#include "ts2vob.h"
#include "ElementaryStream.h"
#include "TimeStamp.h"

class Cutter;

class AC3AudioStream : public ElementaryStream {
    static const int FrameSizeCodeTable[38][3];
    static const int FrameDuration[3];
    static const uint16_t crc_table[256];

    enum {
        CheckSyncFrames = 3,
        MinFrameLength = 2*64,
        MaxFrameLength = 2*1920,
    };

    Cutter &out;
    bool seekSynchronization;
    bool waitingForPTS;
    TimeStamp PTS;

    inline bool isSyncWord(const uint8_t *ptr);
    inline int getSampleRateCode(const uint8_t *ptr);
    inline int getFrameSizeCode(const uint8_t *ptr);
    bool checkCRC(const uint8_t *ptr, int frameSize);
    bool checkSynchronization(unsigned int offset, int count);
    bool trySynchronization();
    inline int getFrameDuration(const uint8_t *ptr);
    inline int getFrameLength(const uint8_t *ptr);
    void newAAUcomplete(unsigned int length);

public:
    AC3AudioStream(Cutter &out);
    virtual void discardAU();
    virtual void buildAUs(bytevec::const_iterator insertPos);
    virtual void endOfStream();
};

#endif
