/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: AccessUnit.h,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef ACCESSUNIT_H
#define ACCESSUNIT_H

#include "ts2vob.h"
#include "TimeStamp.h"

class AccessUnit {
protected:
    bytevec data;

private:
    TimeStamp PTS, DTS;
    int bytesRead;

public:
    AccessUnit(bytevec & data_,
               const TimeStamp & PTS_, const TimeStamp & DTS_) :
        PTS(PTS_), DTS(DTS_), bytesRead(0)
        { data.swap(data_); }
    virtual ~AccessUnit() {}

    const TimeStamp & getPTS() const            { return PTS; }
    const TimeStamp & getDTS() const            { return DTS; }
    void subTimeOffset(int64_t delta)           { PTS -= delta; DTS -= delta; }
    const uint8_t *getData() const              { return &data.front(); }
    int getSize() const                         { return data.size(); }
    const uint8_t *getRestData() const  { return &data[bytesRead]; }
    int getRestSize() const             { return data.size() - bytesRead; }
    int getBytesRead() const                    { return bytesRead; }
    void takeBytes(int count)                   { bytesRead += count; }

    virtual bool startPossible() const = 0;
    virtual void setFirstAfterCutMark() =  0;
    virtual void setEndMark() = 0;
    virtual bool startNewPacketHere() const = 0;
    virtual bool wantsNV_PCK() const = 0;
    virtual bool writeTimeStamp() const = 0;
};

#endif
