/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: Cutter.cc,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#include "Cutter.h"

void Cutter::takeAU(AccessUnit *au)
{
    switch (state) {
        case Skip:
            if ((curIter != endIter) &&
                (au->getDTS() >= curIter->getStartDTS()) &&
                au->startPossible()) {
                au->setFirstAfterCutMark();
                state = Copy;
                break;
            }
            delete au;
            return;

        case Copy:
            if (curIter != endIter) {
                if (au->getDTS() >= curIter->getEndDTS()) {
                    const TimeStamp endDTS = curIter->getEndDTS();
                    ++curIter;
                    if (endIter != curIter) {
                        skippedTime += curIter->getStartDTS() - endDTS;
                        if (au->getDTS() >= curIter->getStartDTS())
                            // no access unit to skip
                            break;
                    }
                    if (lastAU != NULL) {
                        lastAU->setEndMark();
                        out.takeAU(lastAU);
                        lastAU = NULL;
                    }
                    state = Skip;
                    delete au;
                    return;
                }
            }
            break;
    }

    au->subTimeOffset(skippedTime);
    if (lastAU != NULL) {
        out.takeAU(lastAU);
    }
    lastAU = au;
}

void Cutter::flush()
{
    if (lastAU != NULL)
        out.takeAU(lastAU);
    lastAU = NULL;
    out.flush();
}
