/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: ElementaryStream.cc,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#include "ElementaryStream.h"

#include <iostream>

bool ElementaryStream::hasSavedTimeStamp() const
{
    return (!ptsdtsQueue.empty()) && (ptsdtsQueue.front().bytesToRead <= 0);
}

TimeStamp ElementaryStream::getSavedPTS() const
{
    return ptsdtsQueue.front().PTS;
}

TimeStamp ElementaryStream::getSavedDTS() const
{
    return ptsdtsQueue.front().DTS;
}

void ElementaryStream::nextSavedTimeStamp()
{
    ptsdtsQueue.pop_front();
}

void ElementaryStream::decrementBytesToRead(int size)
{
    for (ptsdtsQueueT::iterator i = ptsdtsQueue.begin();
         i != ptsdtsQueue.end();
         ++i) {
        i->bytesToRead -= size;
    }
}

void ElementaryStream::takeFromBuffer(int size, bytevec & dest)
{
    dest.assign(inputBuffer.begin()+size, inputBuffer.end());
    inputBuffer.erase(inputBuffer.begin()+size, inputBuffer.end());
    inputBuffer.swap(dest);
    decrementBytesToRead(size);
}

void ElementaryStream::removeFromBuffer(int size)
{
    inputBuffer.erase(inputBuffer.begin(), inputBuffer.begin()+size);
    decrementBytesToRead(size);
}

void ElementaryStream::clearBuffer()
{
    inputBuffer.clear();
    ptsdtsQueue.clear();
}

void ElementaryStream::addData(const uint8_t *data, int size,
                               bool hasTimeStamp,
                               const TimeStamp & PTS, const TimeStamp & DTS)
{
    if (hasTimeStamp) {
        ptsdtsQueue.push_back(SavedPTSDTS(inputBuffer.size(), PTS, DTS));
    }

    int bytesInBuffer = inputBuffer.size();
    inputBuffer.insert(inputBuffer.end(), data, &data[size]);
    bytevec::const_iterator insertedAt = inputBuffer.begin() + bytesInBuffer;

    buildAUs(insertedAt);
}
