/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: ElementaryStream.h,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef ELEMENTARYSTREAM_H
#define ELEMENTARYSTREAM_H

#include <deque>

#include "ts2vob.h"
#include "TimeStamp.h"

class ElementaryStream {
private:
    struct SavedPTSDTS {
        int bytesToRead;
        TimeStamp PTS;
        TimeStamp DTS;

        SavedPTSDTS(int b, const TimeStamp &p, const TimeStamp &d) :
            bytesToRead(b), PTS(p), DTS(d) {}
    };
    typedef std::deque<SavedPTSDTS> ptsdtsQueueT;
    ptsdtsQueueT ptsdtsQueue;

    bytevec inputBuffer;

    void decrementBytesToRead(int size);

protected:
    const bytevec &buffer;

public:
    ElementaryStream() : buffer(inputBuffer) {}
    virtual ~ElementaryStream() {}
    void addData(const uint8_t *data, int size,
                 bool hasTimeStamp,
                 const TimeStamp & PTS, const TimeStamp & DTS);

    bool hasSavedTimeStamp() const;
    TimeStamp getSavedPTS() const;
    TimeStamp getSavedDTS() const;
    void nextSavedTimeStamp();

    void takeFromBuffer(int size, bytevec & dest);
    void removeFromBuffer(int size);
    void clearBuffer();

    virtual void buildAUs(bytevec::const_iterator insertPos) = 0;
    virtual void discardAU() = 0;
    virtual void endOfStream() = 0;
};

#endif
