/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: MPEGAudioStream.h,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef MPEGAUDIOSTREAM_H
#define MPEGAUDIOSTREAM_H

#include <deque>

#include "ts2vob.h"
#include "ElementaryStream.h"
#include "TimeStamp.h"

class Cutter;

class MPEGAudioStream : public ElementaryStream {
    static const int SamplesPerFrame[3];
    static const int SlotsPerFrame[3];
    static const int BytesPerSlot[3];
    static const int BitRate[2][3][16];
    static const int SampleRate[2][4];

    enum {
        CheckSyncFrames = 3,
    };

    Cutter &out;
    bool seekSynchronization;
    bool waitingForPTS;
    TimeStamp PTS;

    bytevec lastHeader;
    unsigned length, duration;

    inline bool isSyncWord(const uint8_t *ptr, bool lazy);
    inline int getID(const uint8_t *ptr);
    inline int getLayer(const uint8_t *ptr);
    inline int getSamplingFrequencyCode(const uint8_t *ptr);
    inline int getBitRateIndex(const uint8_t *ptr);
    inline int getPaddingBit(const uint8_t *ptr);
    unsigned int getFrameLength(const uint8_t *ptr);
    unsigned int getFrameDuration(const uint8_t *ptr);
    bool checkSynchronization(unsigned int offset, int count);
    bool trySynchronization();
    void newAAUcomplete();

public:
    MPEGAudioStream(Cutter &out_);
    virtual void discardAU();
    virtual void buildAUs(bytevec::const_iterator insertPos);
    virtual void endOfStream();
};

#endif
