/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: MPEGVideoStream.h,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef MPEGVIDEOSTREAM_H
#define MPEGVIDEOSTREAM_H

#include "ts2vob.h"
#include "ElementaryStream.h"
#include "TimeStamp.h"

class VideoAccessUnit;
class Cutter;

class MPEGVideoStream : public ElementaryStream {
private:
    static const int FrameDuration[8];

    Cutter &out;

    bool brokenLink;
    bool waitingForSeqHeader;
    bool PicFound;
    int GOPStartIndex, PICStartIndex, SeqEndStartIndex;
    enum {
        Init,
        GotFirstZero,
        GotSecondZero,
        GotOne
    } state;

    VideoAccessUnit *lastVAU;
    long videoRate;
    int duration; 
    int framesInGOP;
    uint64_t firstPTSinGOP;
    uint64_t firstDTSinGOP;

    void newVAUcomplete(int size);
    bool isSequenceStart() const { return buffer[3] == StartCode::SequenceHeaderCode; }
    int getFrameRateCode() const { return buffer[7] & Bitmask8(4); }
    int getTemporalReference() const { return (buffer[PICStartIndex+4] << 2) | (buffer[PICStartIndex+5] >> 6); }


public:
    MPEGVideoStream(Cutter &out, long video_rate);
    virtual void discardAU();
    virtual void buildAUs(bytevec::const_iterator insertPos);
    virtual void endOfStream();
};

#endif
