/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: OutputPacket.h,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef OUTPUTPACKET_H
#define OUTPUTPACKET_H

#include "ts2vob.h"
#include "TimeStamp.h"

class OutputPacket {
private:
    bytevec data;
    uint8_t stream_id;
    int paddingBytes;

    TimeStamp minSCR, maxSCR;
    TimeStamp PTS, DTS;

    bool write_time_stamp;
    bool wants_NV_PCK;
    bool write_buffer_size;
    int bufferSize;
    bool hasAUEnd;
    int AUEnd;

public:
    OutputPacket(uint8_t id,
                 bytevec &data_, int padding,
                 const TimeStamp & minSCR_, const TimeStamp & maxSCR_,
                 const TimeStamp & PTS_, const TimeStamp & DTS_,
                 bool time_stamp,
                 bool nv_pck,
                 bool puffer, int buffer_size,
                 bool has_au_end, int au_end) :
        stream_id(id),
        paddingBytes(padding),
        minSCR(minSCR_), maxSCR(maxSCR_), PTS(PTS_), DTS(DTS_),
        write_time_stamp(time_stamp), wants_NV_PCK(nv_pck),
        write_buffer_size(puffer), bufferSize(buffer_size),
        hasAUEnd(has_au_end), AUEnd(au_end)
        { data.swap(data_); }

    const uint8_t *getData() const { return &data.front(); }
    int getSize() const { return data.size(); }
    int getPaddingSize() const { return paddingBytes; }

    const TimeStamp & minSCRBase() const { return minSCR; }
    const TimeStamp & maxSCRBase() const { return maxSCR; }
    const TimeStamp & getPTS() const { return PTS; }
    const TimeStamp & getDTS() const { return DTS; }
    bool writeTimeStamp() const { return write_time_stamp; }
    bool writeBufferSize() const { return write_buffer_size; }
    int getBufferSize() const { return bufferSize; }
    bool wantsNV_PCK() const { return wants_NV_PCK; }
    uint8_t getStreamID() const { return stream_id; }
    bool isAUEnd() const { return hasAUEnd; }
    int getAUEnd() const { return AUEnd; }
};

#endif
