/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: PIDHandler.h,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef PIDHANDLER_H
#define PIDHANDLER_H

#include "ts2vob.h"
#include "PESPacket.h"
#include "TimeStamp.h"
#include "Cut.h"
#include "Cutter.h"

class VOBStream;
class Packetizer;
class ElementaryStream;
class TSPacket;

class PIDHandler {
private:
    VOBStream &vob;
    bool ignore;
    Cutter *cutter;
    Packetizer *packetizer;
    ElementaryStream *ES;
    PESPacket pesp;
    int lastCC;
    long videoRate;
    CutConstIter startIter, endIter;

    inline bool ccCorrect(int cc);

public:
    PIDHandler(VOBStream &out, bool ignore_, long video_rate,
               const CutConstIter &start,
               const CutConstIter &end) :
        vob(out), ignore(ignore_), cutter(NULL),
        packetizer(NULL), ES(NULL), lastCC(-1),
        videoRate(video_rate), startIter(start), endIter(end) {}
    ~PIDHandler();
    void takeTSPacket(const TSPacket &packet);
    void endOfStream();
    bool done() { return cutter && cutter->done(); }
};


#endif
