/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: STDBuffer.cc,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#include "STDBuffer.h"

STDBuffer::STDBuffer(int size) :
    bytesFree(size), minSCR(0)
{
}

void STDBuffer::store(int size, const TimeStamp & DTS)
{
    buffer.push_back(DataInBuffer(size, DTS));
    bytesFree -= size;
}

void STDBuffer::releaseUntil(const TimeStamp & SCR)
{
    while (!buffer.empty()) {
        DataInBuffer &d = buffer.front();

        if (d.DTS > SCR)
            break;
        bytesFree += d.size;
        buffer.pop_front();
    }
    minSCR = SCR;
}

TimeStamp STDBuffer::getEarliestTime(int byteNeeded) const
{
    TimeStamp earliestTime = minSCR;

    byteNeeded -= bytesFree;
    for (bufferList::const_iterator i = buffer.begin();
         (byteNeeded > 0) && (i != buffer.end());
         ++i) {
        const DataInBuffer &d = *i;
        
        byteNeeded -= d.size;
        earliestTime = d.DTS;
    }
    return earliestTime;
}
