/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: TSPacket.h,v 1.3 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef TSPACKET_H
#define TSPACKET_H

#include "ts2vob.h"
#include "TimeStamp.h"
class TimeStamp;

class TSPacket {
private:
    const uint8_t *packetData;

    enum {
        SyncByte = 0x47
    };

    enum {
        HeaderSize = 4,
        PacketSize = 188,
    };

public:
    TSPacket() : packetData(NULL) {}
    void setData(const uint8_t *data);

    inline bool syncOk() const;
    inline int getPID() const;
    inline int getCC() const;
    inline bool getTransportError() const;
    inline bool getUnitStart() const;
    inline bool isScrambled() const;
    inline int getAdaptFieldControl() const;
    inline int getAdaptFieldSize() const;
    inline bool hasPCR() const;
    inline TimeStamp getPCRBase() const;
    inline int64_t getPCR() const;
    inline bool hasPayload() const;
    inline bool hasAdaptField() const;
    inline const uint8_t *getPayload() const;
    inline int getPayloadSize() const;
};

inline void TSPacket::setData(const uint8_t *data)
{
    packetData = data;
}

inline bool TSPacket::syncOk() const
{
    return packetData[0] == SyncByte;
}

inline int TSPacket::getPID() const
{
    return ((packetData[1] & Bitmask8(5)) << 8) | packetData[2];
}

inline int TSPacket::getCC() const
{
    return packetData[3] & Bitmask8(4);
}

inline bool TSPacket::getTransportError() const
{
    return (packetData[1] >> 7) & 1;
}

inline bool TSPacket::getUnitStart() const
{
    return (packetData[1] >> 6) & 1;
}

inline bool TSPacket::isScrambled() const
{
    return (packetData[3] >> 6) != 0;
}

inline int TSPacket::getAdaptFieldControl() const
{
    return (packetData[3] >> 4) & Bitmask8(2);
}

inline bool TSPacket::hasPayload() const
{
    return (getAdaptFieldControl() & 1) == 1;
}

inline bool TSPacket::hasAdaptField() const
{
    return (getAdaptFieldControl() & 2) == 2;
}

inline int TSPacket::getAdaptFieldSize() const
{
    if (hasAdaptField())
        return packetData[4]+1;
    else
        return 0;
}

inline bool TSPacket::hasPCR() const
{
    return hasAdaptField() && (((packetData[5] >> 4) & 1) == 1);
}

inline TimeStamp TSPacket::getPCRBase() const
{
    return ((int64_t)packetData[6] << 25) |
        ((int64_t)packetData[7] << 17) | 
        ((int64_t)packetData[8] << 9) |
        ((int64_t)packetData[9] << 1) |
        ((int64_t)packetData[10] >> 7);
}

inline int64_t TSPacket::getPCR() const
{
#if 0
    return (((int64_t)packetData[6] << 25) |
            ((int64_t)packetData[7] << 17) | 
            ((int64_t)packetData[8] << 9) |
            ((int64_t)packetData[9] << 1) |
            ((int64_t)packetData[10] >> 7))*300 +
        ((((int64_t)packetData[10] & 1) << 8) | (int64_t)packetData[11]);
#endif
    return getPCRBase()*300 +
        ((((int64_t)packetData[10] & 1) << 8) | (int64_t)packetData[11]);
}

inline const uint8_t *TSPacket::getPayload() const
{
    return &packetData[4+getAdaptFieldSize()];
}

inline int TSPacket::getPayloadSize() const
{
    if (hasPayload())
        return PacketSize-HeaderSize-getAdaptFieldSize();
    else
        return 0;
}

#endif
