/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: TransportStream.cc,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#include <istream>

#include "TransportStream.h"
#include "TSPacket.h"

TransportStream::TransportStream(std::istream &in_) :
    in(in_),
    bytesInBuffer(0)
{
}

bool TransportStream::fillBuffer()
{
    if (bytesInBuffer > 0)
        memmove(buffer, bufferPtr, bytesInBuffer);

    int count = sizeof(buffer)-bytesInBuffer;
    in.read((char *)&buffer[bytesInBuffer], count);
    bufferPtr = buffer;
    bytesInBuffer += in.gcount();
    return bytesInBuffer == sizeof(buffer);
}

bool TransportStream::seekSync()
{
    if (!fillBuffer())
        return false;

    /* seek Sync-Byte in 5 following packets */
    int pos;
    for (pos=0; pos<PacketSize; pos++) {
        int packet;
        for (packet=0; packet<SyncPackets; packet++)
            if (buffer[packet*PacketSize + pos] != SyncByte)
                break;

        if (packet == SyncPackets)
            break;
    }
    if (pos == PacketSize)
        return false;
    else if (pos == 0)
        return true;

    memmove(buffer, &buffer[pos], sizeof(buffer)-pos);
    in.read((char *)&buffer[sizeof(buffer)-pos], pos);
    return (in.gcount() == pos);
}

bool TransportStream::getPacket(TSPacket &packet)
{
    if (bytesInBuffer < PacketSize) {
        fillBuffer();
        if (bytesInBuffer < PacketSize)
            /* no complete packet in buffer... */
            return false;
    }

    packet.setData(bufferPtr);
    bufferPtr += PacketSize;
    bytesInBuffer -= PacketSize;
    return true;
}
