/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: VideoAccessUnit.cc,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#include "VideoAccessUnit.h"

int VideoAccessUnit::getFrameRateCode() const
{
    return data[7] & Bitmask8(4);
}

int VideoAccessUnit::getTemporalReference() const
{
    return (data[PICStartIndex+4] << 2) | (data[PICStartIndex+5] >> 6);
}

void VideoAccessUnit::setBrokenLink()
{
    if (GOPStartIndex >= 0)
        data[GOPStartIndex+7] |= 0x20;
}

void VideoAccessUnit::setBitRate(int bit_rate_value)
{
    if (isSequenceStart()) {
        data[8] = (bit_rate_value >> 10) & Bitmask8(8);
        data[9] = (bit_rate_value >> 2) & Bitmask8(8);
        data[10] = (data[10] & Bitmask8(6)) |
            ((bit_rate_value & Bitmask8(2)) << 6);
    }
}

void VideoAccessUnit::writeSequenceEndCode()
{
    if (SeqEndStartIndex < 0) {
        SeqEndStartIndex = data.size();
        data.push_back(0x00);
        data.push_back(0x00);
        data.push_back(0x01);
        data.push_back(StartCode::SequenceEndCode);
    }
}

bool VideoAccessUnit::startPossible() const
{
    return isSequenceStart();
}

void VideoAccessUnit::setFirstAfterCutMark()
{
    setBrokenLink();
}

void VideoAccessUnit::setEndMark()
{
    writeSequenceEndCode();
}

bool VideoAccessUnit::isSequenceStart() const
{
    return data[3] == StartCode::SequenceHeaderCode;
}

bool VideoAccessUnit::wantsNV_PCK() const
{
    return isSequenceStart();
}

bool VideoAccessUnit::writeTimeStamp() const {
    return isSequenceStart();
}

bool VideoAccessUnit::startNewPacketHere() const {
    return isSequenceStart();
}
