/*
 * ts2vob - Convert MPEG Transport Stream to Program Stream for DVD-Video
 * Copyright (C) 2005 Martin Winken <martinwi@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/* $Id: ts2vob.h,v 1.2 2006/07/20 16:14:55 martinwi Exp $ */

#ifndef TS2VOB_H
#define TS2VOB_H

#include <vector>
#include <stdint.h>

typedef std::vector<uint8_t> bytevec;

namespace StartCode {
    enum StartCodeValues {
        PictureStartCode = 0x00,
        SequenceHeaderCode = 0xb3,
        ExtensionStartCode = 0xb5,
        SequenceEndCode = 0xb7,
        GroupStartCode = 0xb8,
    };
}

namespace StreamID {
    enum {
        PrivateStream1=0xBD,
        PaddingStream=0xBE,
        PrivateStream2=0xBF,
        AudioStreamMask=0xC0,
        VideoStreamMask=0xE0,
    };
}

enum {
    TicksPerSecond = 90000,
    SCRTicksPerSecond = 300*TicksPerSecond,
};

template<class T> inline T Bitmask(unsigned int bits, unsigned int start = 0)
{
    return ((T(1) << bits)-1) << start;
}

inline uint8_t Bitmask8(unsigned int bits, unsigned int start = 0)
{
    return Bitmask<uint8_t>(bits, start);
}

inline bool IsVideoStreamID(uint8_t id)
{
    return ((id) & Bitmask8(4,4)) == StreamID::VideoStreamMask;
}

inline bool IsAudioStreamID(uint8_t id)
{
    return ((id) & Bitmask8(3,5)) == StreamID::AudioStreamMask;
}

#endif
